#pragma once
#include <kr2_program_api/api_v1/bundles/bundle_class.h>
#include "communication/bridge/proxy/i_program_data_provider.h"
#include "communication/bridge/proxy/i_tool_controller.h"
#include "communication/bridge/proxy/i_secondary_bridge_connector.h"
#include "util/scheduled_executor.h"

#define RETURN_IF(isTerminated) if (isTerminated) CBUN_PCALL_RET_OK;

namespace spin
{
    class Command
    {
    public:
        Command(kr2_program_api::ProgramInterface *api);
        virtual ~Command() = default;

        CBUN_PCALL driveScrew(const std::string uuid, const kr2_program_api::RobotPose &screwPose, kr2_program_api::Number &resultEvaluation);
        CBUN_PCALL loadScrew(const double screwLength_mm, const kr2_program_api::RobotPose &screwPose);
        CBUN_PCALL unloadScrew(const double screwLength_mm, const kr2_program_api::RobotPose &unloadPose);
        CBUN_PCALL changeBit(const int function,
                             const int bitNumber,
                             const kr2_program_api::RobotPose &poseA,
                             const kr2_program_api::RobotPose &poseB,
                             const kr2_program_api::RobotPose &poseC,
                             const bool doTheTwist = true);

    private:
        std::set<Program> programs_;
        std::shared_ptr<ISecondaryBridgeConnector> bridgeConnector_;
        std::shared_ptr<IToolController> toolController_;
        std::shared_ptr<IProgramDataProvider> programDataProvider_;
        kr2_program_api::ProgramInterface *api_;
        kr2_program_api_internals::BusSubscriber subscriber_;
        std::atomic<bool> isProgramTerminated_{false};

        Program findProgram(const UUID &uuid);
        ScheduledExecutor executor_;
        void speedProgramLoop(const UUID &resultUuid, const kr2_program_api::RobotPose &screwPose, const Program &program);
        void onProgramTerminated(const kr2_signal::ProgramTerminated &event);
        bool isTerminated() const;
    };

} // namespace spin